import React, { useState, useRef } from 'react';
import { ACLPolicy, ACLRule } from '../types/acl';
import RuleEditor from './RuleEditor';
import { PlusCircle, ExternalLink, Youtube, X, Upload, AlertCircle, Download } from 'lucide-react';

const ACLEditor: React.FC = () => {
  const [policy, setPolicy] = useState<ACLPolicy>({
    acls: [
      {
        action: 'accept',
        src: [],
        dst: []
      }
    ]
  });
  const [showVideo, setShowVideo] = useState(false);
  const [importError, setImportError] = useState<string | null>(null);
  const fileInputRef = useRef<HTMLInputElement>(null);

  const addRule = () => {
    setPolicy(prev => ({
      ...prev,
      acls: [
        ...prev.acls,
        {
          action: 'accept',
          src: [],
          dst: []
        }
      ]
    }));
  };

  const updateRule = (index: number, updatedRule: ACLRule) => {
    setPolicy(prev => {
      const newAcls = [...prev.acls];
      newAcls[index] = updatedRule;
      return { ...prev, acls: newAcls };
    });
  };

  const removeRule = (index: number) => {
    setPolicy(prev => {
      const newAcls = prev.acls.filter((_, i) => i !== index);
      return { ...prev, acls: newAcls };
    });
  };

  const handleImportClick = () => {
    if (fileInputRef.current) {
      fileInputRef.current.click();
    }
  };

  const handleFileChange = (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (!file) return;

    const reader = new FileReader();
    reader.onload = (e) => {
      try {
        const content = e.target?.result as string;
        const importedPolicy = JSON.parse(content);
        
        // Basic validation
        if (!importedPolicy.acls || !Array.isArray(importedPolicy.acls)) {
          throw new Error('Invalid ACL format: missing or invalid "acls" array');
        }
        
        // More detailed validation could be added here
        
        setPolicy(importedPolicy);
        setImportError(null);
      } catch (error) {
        setImportError(`Failed to import ACL: ${error instanceof Error ? error.message : 'Unknown error'}`);
      }
    };
    reader.onerror = () => {
      setImportError('Failed to read the file');
    };
    reader.readAsText(file);
    
    // Reset the file input
    if (event.target) {
      event.target.value = '';
    }
  };

  const handleDownload = () => {
    // Create a blob with the JSON data
    const jsonString = JSON.stringify(policy, null, 2);
    const blob = new Blob([jsonString], { type: 'application/json' });
    
    // Create a URL for the blob
    const url = URL.createObjectURL(blob);
    
    // Create a temporary anchor element and trigger download
    const a = document.createElement('a');
    a.href = url;
    a.download = 'tailscale-acl.json';
    document.body.appendChild(a);
    a.click();
    
    // Clean up
    document.body.removeChild(a);
    URL.revokeObjectURL(url);
  };

  const dismissError = () => {
    setImportError(null);
  };

  return (
    <div className="container mx-auto p-4">
      <div className="flex justify-between items-center mb-6 border-b border-[#333] pb-4">
        <div className="flex items-center">
          <h1 className="text-xl font-medium">Tailscale ACL Editor</h1>
        </div>
        <div className="flex items-center">
          <button
            onClick={handleDownload}
            className="text-[#999] hover:text-white font-normal mr-4"
          >
            Download
          </button>
          <div className="w-8 h-8 rounded-full bg-[#333] flex items-center justify-center text-white">
            <span>U</span>
          </div>
        </div>
      </div>
      
      {showVideo && (
        <div className="fixed inset-0 bg-black bg-opacity-75 flex items-center justify-center z-50 p-4">
          <div className="relative w-full max-w-4xl">
            <button 
              onClick={() => setShowVideo(false)}
              className="absolute -top-10 right-0 text-white hover:text-gray-300"
            >
              <X size={24} />
            </button>
            <div className="relative pb-[56.25%] h-0">
              <iframe 
                className="absolute top-0 left-0 w-full h-full rounded-lg"
                src="https://www.youtube.com/embed/Jn8_Sh4r8d4" 
                title="Tailscale ACL Tutorial"
                allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" 
                allowFullScreen
              ></iframe>
            </div>
          </div>
        </div>
      )}
      
      {importError && (
        <div className="bg-red-900 border border-red-700 text-white px-4 py-3 rounded relative mb-4">
          <span className="flex items-center gap-2">
            <AlertCircle size={18} />
            {importError}
          </span>
          <button 
            className="absolute top-0 bottom-0 right-0 px-4"
            onClick={dismissError}
          >
            <X size={18} />
          </button>
        </div>
      )}
      
      <div className="flex justify-between items-center mb-6">
        <div>
          <p className="text-[#999] mb-2">Manage access control rules for your Tailnet. <a href="https://tailscale.com/kb/1018/acls" className="text-[#4f8cc9] hover:text-[#5f9cd9]">Learn more</a></p>
        </div>
        <div>
          <button 
            onClick={addRule}
            className="bg-[#333] hover:bg-[#444] text-white px-4 py-2 rounded-md mr-2"
          >
            Add Rule
          </button>
          
          <button 
            onClick={handleImportClick}
            className="bg-[#333] hover:bg-[#444] text-white px-4 py-2 rounded-md"
          >
            Import ACL
          </button>
          <input 
            type="file" 
            ref={fileInputRef}
            onChange={handleFileChange}
            accept=".json"
            className="hidden"
          />
        </div>
      </div>
      
      <div className="grid grid-cols-1 gap-6 mb-6">
        {policy.acls.map((rule, index) => (
          <RuleEditor 
            key={index}
            rule={rule}
            onChange={(updatedRule) => updateRule(index, updatedRule)}
            onRemove={() => removeRule(index)}
            ruleNumber={index + 1}
          />
        ))}
      </div>
      
      <div className="mt-8">
        <h2 className="text-lg font-medium mb-2">ACL JSON Preview</h2>
        <pre className="bg-[#222] text-[#4f8cc9] p-4 rounded-md overflow-auto">
          {JSON.stringify(policy, null, 2)}
        </pre>
      </div>
    </div>
  );
};

export default ACLEditor;
