import React, { useState } from 'react';
import { DestinationEntry, DestinationType } from '../types/acl';
import { PlusCircle, X, AlertCircle } from 'lucide-react';
import { isValidDestinationValue } from '../utils/validation';

interface DestinationSelectorProps {
  destinations: DestinationEntry[];
  onChange: (destinations: DestinationEntry[]) => void;
}

const DestinationSelector: React.FC<DestinationSelectorProps> = ({ destinations, onChange }) => {
  const [newDestType, setNewDestType] = useState<DestinationType>('any');
  const [newDestValue, setNewDestValue] = useState<string>('*');
  const [newDestPorts, setNewDestPorts] = useState<string>('*');
  const [validationError, setValidationError] = useState<string | null>(null);

  const destTypes: { value: DestinationType; label: string }[] = [
    { value: 'any', label: 'Any (*)' },
    { value: 'user', label: 'User' },
    { value: 'group', label: 'Group' },
    { value: 'ip', label: 'Tailscale IP' },
    { value: 'subnet', label: 'Subnet CIDR' },
    { value: 'host', label: 'Host' },
    { value: 'tag', label: 'Tag' },
    { value: 'autogroup', label: 'Autogroup' }
  ];

  const validateAndAddDestination = () => {
    // Clear previous error
    setValidationError(null);
    
    if (newDestType === 'any') {
      onChange([...destinations, { type: 'any', value: '*', ports: '*' }]);
      return;
    }
    
    if (!newDestValue.trim()) {
      setValidationError('Value cannot be empty');
      return;
    }
    
    // Validate based on destination type and port
    if (!isValidDestinationValue(newDestType, newDestValue, newDestPorts)) {
      // Determine if the error is with the destination or port
      if (newDestPorts !== '*' && !isValidDestinationValue(newDestType, newDestValue, '*')) {
        setValidationError(`Invalid ${newDestType} format`);
      } else {
        setValidationError('Invalid port format');
      }
      return;
    }
    
    // If validation passes, add the destination
    onChange([
      ...destinations, 
      { 
        type: newDestType, 
        value: newDestValue.trim(), 
        ports: newDestPorts.trim() || '*'
      }
    ]);
    setNewDestValue('');
    setNewDestPorts('*');
  };

  const removeDestination = (index: number) => {
    onChange(destinations.filter((_, i) => i !== index));
  };

  const handleDestTypeChange = (e: React.ChangeEvent<HTMLSelectElement>) => {
    const type = e.target.value as DestinationType;
    setNewDestType(type);
    setValidationError(null);
    
    // Set default value based on type
    if (type === 'any') {
      setNewDestValue('*');
    } else if (type === 'group') {
      setNewDestValue('group:');
    } else if (type === 'tag') {
      setNewDestValue('tag:');
    } else if (type === 'autogroup') {
      setNewDestValue('autogroup:');
    } else {
      setNewDestValue('');
    }
  };

  const getDestinationDisplay = (dest: DestinationEntry): string => {
    if (dest.type === 'any') return '*:*';
    return `${dest.value}:${dest.ports}`;
  };

  return (
    <div>
      <div className="flex flex-wrap gap-2 mb-2">
        {destinations.map((dest, index) => (
          <div 
            key={index} 
            className="flex items-center bg-[#2a5545] text-white px-3 py-1 rounded-full"
          >
            <span className="mr-1">{getDestinationDisplay(dest)}</span>
            <button 
              onClick={() => removeDestination(index)}
              className="text-[#ccc] hover:text-white"
            >
              <X size={14} />
            </button>
          </div>
        ))}
        {destinations.length === 0 && (
          <div className="text-[#999] italic">No destinations added (required)</div>
        )}
      </div>

      {validationError && (
        <div className="flex items-center text-red-500 text-sm mb-2">
          <AlertCircle size={14} className="mr-1" />
          {validationError}
        </div>
      )}

      <div className="flex gap-2">
        <select
          className="p-2 border border-[#444] bg-[#333] text-white rounded-md"
          value={newDestType}
          onChange={handleDestTypeChange}
        >
          {destTypes.map(type => (
            <option key={type.value} value={type.value}>{type.label}</option>
          ))}
        </select>
        
        {newDestType !== 'any' && (
          <>
            <input
              type="text"
              className={`flex-1 p-2 border ${validationError && validationError.includes(newDestType) ? 'border-red-500' : 'border-[#444]'} bg-[#333] text-white rounded-md`}
              value={newDestValue}
              onChange={(e) => {
                setNewDestValue(e.target.value);
                setValidationError(null);
              }}
              placeholder={`Enter ${newDestType} value`}
            />
            
            <input
              type="text"
              className={`w-32 p-2 border ${validationError && validationError.includes('port') ? 'border-red-500' : 'border-[#444]'} bg-[#333] text-white rounded-md`}
              value={newDestPorts}
              onChange={(e) => {
                setNewDestPorts(e.target.value);
                setValidationError(null);
              }}
              placeholder="Ports"
            />
          </>
        )}
        
        <button
          onClick={validateAndAddDestination}
          className="bg-[#4f8cc9] hover:bg-[#5f9cd9] text-white px-3 py-2 rounded-md flex items-center"
        >
          <PlusCircle size={18} />
        </button>
      </div>
      
      <div className="text-xs text-[#999] mt-1">
        {newDestType === 'user' && "Example: user@example.com"}
        {newDestType === 'group' && "Example: group:engineering"}
        {newDestType === 'ip' && "Example: 100.101.102.103"}
        {newDestType === 'subnet' && "Example: 192.168.1.0/24"}
        {newDestType === 'host' && "Example: my-host"}
        {newDestType === 'tag' && "Example: tag:production"}
        {newDestType === 'autogroup' && "Example: autogroup:internet"}
        {newDestType !== 'any' && "Ports: * (any), 22 (single), 80,443 (multiple), 1000-2000 (range)"}
      </div>
    </div>
  );
};

export default DestinationSelector;
