import React from 'react';
import { ACLRule, Protocol, SourceEntry, DestinationEntry } from '../types/acl';
import SourceSelector from './SourceSelector';
import DestinationSelector from './DestinationSelector';
import { Trash2 } from 'lucide-react';

interface RuleEditorProps {
  rule: ACLRule;
  onChange: (rule: ACLRule) => void;
  onRemove: () => void;
  ruleNumber: number;
}

const RuleEditor: React.FC<RuleEditorProps> = ({ rule, onChange, onRemove, ruleNumber }) => {
  const protocols: Protocol[] = ['tcp', 'udp', 'icmp', 'sctp', 'igmp', 'esp', 'ah', 'gre', 'ipip', 'dccp', 'all'];

  const handleProtoChange = (e: React.ChangeEvent<HTMLSelectElement>) => {
    const value = e.target.value as Protocol | '';
    onChange({
      ...rule,
      proto: value === '' ? undefined : value as Protocol
    });
  };

  const handleSourcesChange = (sources: SourceEntry[]) => {
    onChange({
      ...rule,
      src: sources
    });
  };

  const handleDestinationsChange = (destinations: DestinationEntry[]) => {
    onChange({
      ...rule,
      dst: destinations
    });
  };

  return (
    <div className="border border-[#333] rounded-lg p-4 bg-[#222] shadow-sm">
      <div className="flex justify-between items-center mb-4">
        <h3 className="text-lg font-medium">Rule #{ruleNumber}</h3>
        <button 
          onClick={onRemove}
          className="text-[#999] hover:text-white"
          title="Remove rule"
        >
          <Trash2 size={18} />
        </button>
      </div>

      <div className="grid grid-cols-1 md:grid-cols-2 gap-4 mb-4">
        <div>
          <label className="block text-sm font-medium text-[#999] mb-1">Action</label>
          <select 
            className="w-full p-2 border border-[#444] bg-[#333] text-white rounded-md"
            value={rule.action}
            disabled
          >
            <option value="accept">accept</option>
          </select>
          <p className="text-xs text-[#777] mt-1">
            Tailscale only supports 'accept' actions (deny by default)
          </p>
        </div>

        <div>
          <label className="block text-sm font-medium text-[#999] mb-1">Protocol (optional)</label>
          <select 
            className="w-full p-2 border border-[#444] bg-[#333] text-white rounded-md"
            value={rule.proto || ''}
            onChange={handleProtoChange}
          >
            <option value="">All protocols</option>
            {protocols.map(proto => (
              <option key={proto} value={proto}>{proto}</option>
            ))}
          </select>
        </div>
      </div>

      <div className="mb-4">
        <label className="block text-sm font-medium text-[#999] mb-1">Sources (src)</label>
        <SourceSelector 
          sources={rule.src}
          onChange={handleSourcesChange}
        />
      </div>

      <div className="mb-4">
        <label className="block text-sm font-medium text-[#999] mb-1">Destinations (dst)</label>
        <DestinationSelector 
          destinations={rule.dst}
          onChange={handleDestinationsChange}
        />
      </div>
    </div>
  );
};

export default RuleEditor;
