import React, { useState } from 'react';
import { SourceEntry, SourceType } from '../types/acl';
import { PlusCircle, X, AlertCircle } from 'lucide-react';
import { isValidSourceValue } from '../utils/validation';

interface SourceSelectorProps {
  sources: SourceEntry[];
  onChange: (sources: SourceEntry[]) => void;
}

const SourceSelector: React.FC<SourceSelectorProps> = ({ sources, onChange }) => {
  const [newSourceType, setNewSourceType] = useState<SourceType>('any');
  const [newSourceValue, setNewSourceValue] = useState<string>('*');
  const [validationError, setValidationError] = useState<string | null>(null);

  const sourceTypes: { value: SourceType; label: string }[] = [
    { value: 'any', label: 'Any (*)' },
    { value: 'user', label: 'User' },
    { value: 'group', label: 'Group' },
    { value: 'ip', label: 'Tailscale IP' },
    { value: 'subnet', label: 'Subnet CIDR' },
    { value: 'host', label: 'Host' },
    { value: 'tag', label: 'Tag' },
    { value: 'autogroup', label: 'Autogroup' }
  ];

  const validateAndAddSource = () => {
    // Clear previous error
    setValidationError(null);
    
    if (newSourceType === 'any') {
      onChange([...sources, { type: 'any', value: '*' }]);
      return;
    }
    
    if (!newSourceValue.trim()) {
      setValidationError('Value cannot be empty');
      return;
    }
    
    // Validate based on source type
    if (!isValidSourceValue(newSourceType, newSourceValue)) {
      setValidationError(`Invalid ${newSourceType} format`);
      return;
    }
    
    // If validation passes, add the source
    onChange([...sources, { type: newSourceType, value: newSourceValue.trim() }]);
    setNewSourceValue('');
  };

  const removeSource = (index: number) => {
    onChange(sources.filter((_, i) => i !== index));
  };

  const handleSourceTypeChange = (e: React.ChangeEvent<HTMLSelectElement>) => {
    const type = e.target.value as SourceType;
    setNewSourceType(type);
    setValidationError(null);
    
    // Set default value based on type
    if (type === 'any') {
      setNewSourceValue('*');
    } else if (type === 'group') {
      setNewSourceValue('group:');
    } else if (type === 'tag') {
      setNewSourceValue('tag:');
    } else if (type === 'autogroup') {
      setNewSourceValue('autogroup:');
    } else {
      setNewSourceValue('');
    }
  };

  const getSourceDisplay = (source: SourceEntry): string => {
    if (source.type === 'any') return '*';
    if (source.type === 'group') return source.value;
    if (source.type === 'tag') return source.value;
    if (source.type === 'autogroup') return source.value;
    return source.value;
  };

  return (
    <div>
      <div className="flex flex-wrap gap-2 mb-2">
        {sources.map((source, index) => (
          <div 
            key={index} 
            className="flex items-center bg-[#2a4562] text-white px-3 py-1 rounded-full"
          >
            <span className="mr-1">{getSourceDisplay(source)}</span>
            <button 
              onClick={() => removeSource(index)}
              className="text-[#ccc] hover:text-white"
            >
              <X size={14} />
            </button>
          </div>
        ))}
        {sources.length === 0 && (
          <div className="text-[#999] italic">No sources added (required)</div>
        )}
      </div>

      {validationError && (
        <div className="flex items-center text-red-500 text-sm mb-2">
          <AlertCircle size={14} className="mr-1" />
          {validationError}
        </div>
      )}

      <div className="flex gap-2">
        <select
          className="p-2 border border-[#444] bg-[#333] text-white rounded-md"
          value={newSourceType}
          onChange={handleSourceTypeChange}
        >
          {sourceTypes.map(type => (
            <option key={type.value} value={type.value}>{type.label}</option>
          ))}
        </select>
        
        {newSourceType !== 'any' && (
          <input
            type="text"
            className={`flex-1 p-2 border ${validationError ? 'border-red-500' : 'border-[#444]'} bg-[#333] text-white rounded-md`}
            value={newSourceValue}
            onChange={(e) => {
              setNewSourceValue(e.target.value);
              setValidationError(null);
            }}
            placeholder={`Enter ${newSourceType} value`}
          />
        )}
        
        <button
          onClick={validateAndAddSource}
          className="bg-[#4f8cc9] hover:bg-[#5f9cd9] text-white px-3 py-2 rounded-md flex items-center"
        >
          <PlusCircle size={18} />
        </button>
      </div>
      
      <div className="text-xs text-[#999] mt-1">
        {newSourceType === 'user' && "Example: user@example.com"}
        {newSourceType === 'group' && "Example: group:engineering"}
        {newSourceType === 'ip' && "Example: 100.101.102.103"}
        {newSourceType === 'subnet' && "Example: 192.168.1.0/24"}
        {newSourceType === 'host' && "Example: my-host"}
        {newSourceType === 'tag' && "Example: tag:production"}
        {newSourceType === 'autogroup' && "Example: autogroup:member"}
      </div>
    </div>
  );
};

export default SourceSelector;
